/***************************************************/
/*! \class FDNJot4LP
    \STK order 4 single input, single output feedback
    \delay network, with each delay line cascaded by 
    \a one-pole lowpass filter. There is also a tone
    \correction filter.

    This private class implements an order 4 SISO FDN
    with B = [1,1,1,1]^T, C = [1,1,1,1]^T, and feedback
    matrix A = A_4, where A_N = gA*(I_N - (2/N)*u_N*(u_N)^T).
    (I_N denotes the N by N identity matrix, and
    u_N = [1,...,1]^T, an N-vector of ones.) Each delay is
    cascaded by a lowpass filter according to the
    difference equation:
         y_i(n) = g_i(1-a_i)*x_i(n) + a_i*y_i(n-1)
    where the subscript i = {1,2,3,4} denotes each of the
    4 delay lines. A tone correction filter is applied
    to the FDN output and summed with the input sample
    to produce the final system output.

    You set the reverberation time at DC and at half the
    sampling rate, the gain coefficient gA of the A matrix,
    and the delay line lengths of each of the 4 delay lines
    during instantiation. The default constructor sets up a 
    0-sample length for each delay line with gA = 0.0, g_i = 1.0, 
    and a_i = 0.0 for all i and no tone correction (resulting in 
    no delay, feedback, or filtering).

    Music 421, Spring Quarter 2003-2004
*/
/***************************************************/

#if !defined(__FDNJOT4LP_H)
#define __FDNJOT4LP_H

#include "Delay.h"
#include "Filter.h"

class FDNJot4LP
{
public:

  //! Constructor which sets the t60 reverb times at DC and at half
  //! the sampling rate, the gain coefficient of the feedback matrix, 
  //! and the FDN delay line lengths
  FDNJot4LP(MY_FLOAT RTatDC, MY_FLOAT RTatPI, MY_FLOAT gainAVal, long *delayL);

  //! Class destructor.
  ~FDNJot4LP();

  //! Clears the internal states of the FDN delay lines and filters.
  void clear();

  //! Input one sample to the lowpassed FDN and return one output.
  MY_FLOAT tick(MY_FLOAT sample);

private:
  MY_FLOAT gA; // gain coefficient gA for the A feedback matrix

  MY_FLOAT dl_inputs[4]; // delay line inputs
  MY_FLOAT dl_outputs[4]; // delay line outputs (also lp filter inputs)
  MY_FLOAT lp_outputs[4]; // lowpass filter outputs
  MY_FLOAT sum_lp_outputs; // sum of lowpass filter outputs
  MY_FLOAT tc_output; // output of tone correction filter

  Delay *delayline[4]; // array of pointers to delay line objects
  Filter *lpFilter[4]; // array of pointers to lowpass filter object
  Filter *tcFilter; // pointer to tone correction filter
};

#endif

