/* 
   Main test program for using FDNJot4LP.cpp, a very basic order 4 
   SISO feedback delay network with lowpass filters cascaded after
   each delay line and a tone correction filter.
   
   The input soundfile will determine the sampling rate of the 
   processing. For now, the LP FDN parameters (reverb time
   at DC and pi, matrix gain, and the four delay-line lengths) are 
   specified in the body of the main program and not as arguments.
   The FDN's output is normalized to 1.0 before saving the reverberated
   result into a soundfile.
 
   Example: testFDNJot4LP i.wav o.wav 5
     This reads in the soundfile i.wav, filters it through the
     order 4 lowpass feedback delay network, and saves the 5 second
     long output into the soundfile o.wav.

   Music 421, Spring Quarter 2003-2004
*/

#include "FileWvIn.h"  // STK-4 sound input support 
#include "FileWvOut.h" // STK-4 sound output support
#include "Stk.h" // for dealing with global sampling rate
#include "FDNJot4LP.h" 
#include <math.h> // for fabs()
#include <stdlib.h> // for atof(()

int main(int argc, char *argv[])
{
  char *outputName; // output soundfile name
  MY_FLOAT computeSeconds = 2; // seconds to compute

  long M[4] = {113,337,1051,4799}; // delay line lengths in samples
  MY_FLOAT gCoeff = 1.0; // gain coefficient for the A feedback matrix
  MY_FLOAT t60_DC = 2.0; // reverberation time at DC
  MY_FLOAT t60_pi = 1.0; // reverberation time at half the sampling rate

  if (argc<2) {
    printf("You must give at minimum an argument for the soundfile input\n");
  }
  if (argc>2) {
    outputName = argv[2]; 
  } else {
    outputName = argv[0]; // default: take the name of this .cpp file
  }
  if (argc>3) {
     computeSeconds = atof(argv[3]);
  }

  FileWvIn input_temp(argv[1]); // reads input
  MY_FLOAT fs = input_temp.getFileRate(); // sampling rate of input soundfile
  Stk::setSampleRate(fs); // set sampling rate to that of input
  FileWvIn input(argv[1]); // rereads input at new sampling rate
  FileWvOut output(outputName); // creates audio file output

 
  MY_FLOAT maxValue = 0.0;
  long nSamps = (long) (fs*computeSeconds);
  MY_FLOAT forOutput[nSamps]; // output buffer

  // creates order 4 LP FDN
  FDNJot4LP *simpleFDN4LP = new FDNJot4LP(t60_DC,t60_pi,gCoeff,M); 

  MY_FLOAT inputSamp, outputSamp, revOut, absRevOut;
  for (long i=0;i<nSamps;i++)   {
    inputSamp = input.tick();
    revOut = simpleFDN4LP->tick(inputSamp);
    absRevOut = fabs(revOut);
    forOutput[i] = revOut; // save FDN output samples
    if (absRevOut > maxValue) maxValue = absRevOut;
    //printf("%ld   %f\n",i,filtResult); // print out to terminal
  }

  for (long i=0;i<nSamps;i++) {
    outputSamp = forOutput[i]/maxValue; // scale appropriately
    output.tick(outputSamp); // output samples to a soundfile
  }

}
